#!/usr/bin/env python
import os
import json
import shutil
import unittest
import requests
import pytest
import tempfile
from pathlib import Path
from eossr import ROOT_DIR
from eossr.api.zenodo import ZenodoAPI, get_zenodo_records, get_record, Record


eossr_test_lib_id = 930570  # test library in sandbox (owner: T. Vuillaume)


class TestZenodoApiSandbox(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestZenodoApiSandbox, self).__init__(*args, **kwargs)
        self.token = 'FakeToken'
        self.zenodo = ZenodoAPI(access_token=self.token,
                                sandbox=True,
                                proj_root_dir=ROOT_DIR)

    def test_initialization_sandbox(self):
        assert isinstance(self.zenodo, ZenodoAPI)
        assert self.zenodo.api_url == 'https://sandbox.zenodo.org/api'
        assert self.zenodo.access_token == self.token
        assert self.zenodo.root_dir == Path(ROOT_DIR)
        assert self.zenodo.path_codemeta_file == self.zenodo.root_dir.joinpath('codemeta.json')
        assert self.zenodo.path_zenodo_file == self.zenodo.root_dir.joinpath('.zenodo.json')
        assert isinstance(self.zenodo.root_dir, (Path, str))

    def test_fetch_community_entries(self):
        community_entries = self.zenodo.fetch_community_entries()
        assert isinstance(community_entries, requests.models.Response)

    def test_conversion_codemeta2zenodo_and_search_zenodo_file(self):
        token = 'FakeToken'
        z = ZenodoAPI(access_token=token,
                      sandbox=False,
                      proj_root_dir=ROOT_DIR)

        assert z.path_codemeta_file.exists()
        z.conversion_codemeta2zenodo()

        assert z.path_zenodo_file.exists()
        assert z.path_zenodo_file == z.root_dir.joinpath('.zenodo.json')
        with open(z.path_zenodo_file) as f:
            zenodo_meta = json.load(f)

        os.remove(z.path_zenodo_file)


class TestZenodoAPINoToken(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TestZenodoAPINoToken, self).__init__(*args, **kwargs)
        self.token = 'FakeToken'
        self.zenodo = ZenodoAPI(access_token=self.token,
                                sandbox=False,
                                proj_root_dir=ROOT_DIR)

    def test_initialization(self):
        assert isinstance(self.zenodo, ZenodoAPI)
        assert self.zenodo.api_url == 'https://zenodo.org/api'
        assert self.zenodo.access_token == self.token
        assert self.zenodo.root_dir == Path(ROOT_DIR)
        assert isinstance(self.zenodo.root_dir, (Path, str))


@pytest.mark.skipif(os.getenv('SANDBOX_ZENODO_TOKEN') is None, reason="SANDBOX_ZENODO_TOKEN not defined")
class TestZenodoAPIToken(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestZenodoAPIToken, self).__init__(*args, **kwargs)
        self.root_dir = Path(ROOT_DIR)
        self.token = os.getenv('SANDBOX_ZENODO_TOKEN')
        self.zenodo = ZenodoAPI(access_token=self.token, sandbox=True, proj_root_dir=ROOT_DIR)

    def test_init(self):
        self.zenodo.access_token == os.getenv('SANDBOX_ZENODO_TOKEN')

    def test_get_user_records(self):
        records = self.zenodo.get_user_records()
        assert isinstance(records[0], Record)

    def test_fetch_user_entries(self):
        self.zenodo.fetch_user_entries()

    def test_create_erase_new_entry(self):
        create_new_entry = self.zenodo.create_new_entry()
        assert isinstance(create_new_entry, requests.models.Response)
        record_id = create_new_entry.json()['id']
        erase_entry = self.zenodo.erase_entry(record_id)
        assert isinstance(erase_entry, requests.models.Response)

    def test_upload_package(self):
        # prepare upload in a tmpdir
        with tempfile.TemporaryDirectory() as tmpdirname:
            print(f"tmpdir {tmpdirname}")
            shutil.copy(Path(ROOT_DIR).joinpath('codemeta.json'), tmpdirname)
            _, filename = tempfile.mkstemp(dir=tmpdirname)
            Path(filename).write_text('Hello from eossr unit tests')
            self.zenodo.set_root_dir(tmpdirname)

            # create new record
            new_record_id = self.zenodo.upload_dir_content(publish=False)
            self.zenodo.erase_entry(new_record_id)
            print(f"{new_record_id} created and deleted")

            # update existing one
            new_record_id = self.zenodo.upload_dir_content(record_id=eossr_test_lib_id, publish=False)
            self.zenodo.erase_entry(new_record_id)
            print(f"{new_record_id} created and deleted")

    def test_find_similar_records_sandbox(self):
        self.zenodo.parameters.update({'size': 100})
        existing_record = Record.from_id(939075, sandbox=True)  # One of the copies of eossr
        assert len(self.zenodo.find_similar_records(existing_record)) > 0
        not_existing_record = Record.from_id(767507, sandbox=True)  # Sandbox E.GARCIA ZenodoCI_vTest Records
        assert self.zenodo.find_similar_records(not_existing_record) == []


def test_get_zenodo_records():
    l = get_zenodo_records('ESCAPE template project')
    assert len(l) > 1
    all_dois = [r.data['doi'] for r in l]
    assert '10.5281/zenodo.4923992' in all_dois


@pytest.mark.xfail(raises=ValueError)
def test_get_record_42():
    get_record(42)


@pytest.fixture
def test_get_record_4923992():
    record = get_record(4923992)
    assert record.data['conceptdoi'] == '10.5281/zenodo.3572654'
    return record


def test_record(test_get_record_4923992):
    record = test_get_record_4923992
    record.print_info()
    codemeta = record.get_codemeta()
    assert isinstance(codemeta, dict)
    assert codemeta['name'] == 'ESCAPE template project'
    record.get_mybinder_url()


def test_get_record_sandbox():
    record = get_record(520735, sandbox=True)
    assert record.data['doi'] == '10.5072/zenodo.520735'


def test_write_record_zenodo(test_get_record_4923992, tmpdir):
    record = test_get_record_4923992
    record.write_zenodo(filename=tmpdir/'.zenodo.json')
    with open(tmpdir/'.zenodo.json') as file:
        json_dict = json.load(file)
    assert json_dict['conceptdoi'] == '10.5281/zenodo.3572654'
